/***************** DDS_SMPL.C: ******************

Simple universal 32 bit driver for DDS chips,
serial data, LSB first. Active pins: serial
data, serial clock, shift enable, transfer pulse.

Interface:
 -printer port pin2: Serial Data (active high)
               pin3: Serial Clock (active pos.edge)
               pin4: Shift Enable (active high)
               pin5: Transfer Pulse (active high)
*************************************************/

#include <stdio.h>
#include <dos.h>

/* printer port address: */
#define LPT1 0x378
#define LPT2 0x278

/* printer port pinout: */
#define SD 0x01
#define SCLK 0x02
#define SH_EN 0x04
#define TXFR 0x08

/* global variables: */
int lpt_base = LPT2;  /* change this as required */

/* low level control of the printer port: */
void clear_port ()
{
  outportb (lpt_base, 0);
}

void set_data (int d)
/* setup data pin: */
{
  if (d)
    outportb (lpt_base,inportb(lpt_base)|SD);  /* set bit, or... */
  else
    outportb (lpt_base,inportb(lpt_base)&~SD); /* clear bit. */
}

void clock_one()
/* generate one clock pulse: */
{
  /* generate one clock pulse:*/
  outportb (lpt_base,inportb(lpt_base)&~SCLK);  /* clock low.. */
  outportb (lpt_base,inportb(lpt_base)|SCLK);   /* and high.. */
  outportb (lpt_base,inportb(lpt_base)&~SCLK);  /* and back to low. */
}

void enable_shift (int e)
/* control the state of the 'shift_enable' line: */
{
  if (e)
    outportb (lpt_base,SH_EN);   /* set bit */
  else
    outportb (lpt_base,0);       /* clear port. */
}

void txfr_data ()
/* TXFR is active high, controls the Hop_Clk line of Q2220. */
{
  outportb (lpt_base,inportb(lpt_base)|TXFR);    /* set bit... */
  outportb (lpt_base,inportb(lpt_base)&(~TXFR));   /* and reset. */
}

void set_freq (long N)
{
  /* shift out 32 bits of DDS phase increment, LSB first: */
  int i,d;

  enable_shift (1);
  for (i=0; i<32; i++) {/* for all the 32 data bits: */
    d = N & 0x00000001; /* isolate LSB... */
    set_data (d);       /* setup data bit... */
    clock_one();        /* ...clock... */
    N = N >> 1;         /* shift frequency control word right. */
  }
  enable_shift (0);
  txfr_data();
}

main ()
{
  /* simple main program: */
  unsigned long N;    /* DDS phase increment */
  double fref = 50.0; /* put exact value [in MHz] here for calibration */
  double f;           /* if 'fref' is in kHz, this can be kHz too. */
  char buff[160];     /* accept a looong keyboard entry here ... */

  puts ("enter f<=0 to quit.");
  clear_port();
  do {
    printf ("f[MHz]: ");
    gets (buff); sscanf (buff,"%lf",&f);
    N = (long)(f/fref*4294967296.0);  /* the 'double' is 2^22 */
    set_freq (N);
  } while (f > 0);
  clear_port();
  return (0);
}